/*
 * @Description: vlp16驱动测试
 * @Author: yct
 * @Date: 2021-07-26 14:11:26
 * @LastEditTime: 2021-07-30 16:25:35
 * @LastEditors: yct
 */

#include <iostream>
#include <fstream>

#include "../velodyne_lidar/velodyne_driver/input.h"
#include "../velodyne_lidar/velodyne_driver/rawdata.h"
bool velo_driver_test()
{
    velodyne_driver::InputSocket sock;
    if (!sock.init("", 2368))
    {
        std::cout << "sock init failed" << std::endl;
        return false;
    }

    velodyne_rawdata::RawData data;
    data.setup("VLP16", "/home/youchen/extra_space/chutian/measure/puai_wj_2021/setting/VLP16db.yaml", 1.2, 0.1, 0, 360);

    int count = 0;
    unsigned char pkt[1206];
    std::vector<Lidar_point> t_point;
    while (count++ < 50000)
    {
        int ret = sock.getPacket(pkt);
        if (ret != 0)
            std::cout << "get pack: " << ret << std::endl;
        else
        {
            // t_point.clear();
            // memset(&pkt[1205], 0, 1);
            // printf("%s\n", pkt);
            data.unpack(pkt, t_point);
            std::cout << "point size: " << t_point.size() << std::endl;
            if(t_point.size() > 30000)
            {
                std::ofstream out("/home/youchen/extra_space/chutian/measure/puai_wj_2021/build/point.txt");
                if(out.is_open())
                {
                    for (size_t i = 0; i < t_point.size(); i++)
                    {
                        char buf[255] = {0};
                        sprintf(buf, "%.3f %.3f %.3f %.6f\n", t_point[i].x, t_point[i].y, t_point[i].z, t_point[i].timestamp);
                        out << buf;
                    }
                    out.close();
                    return true;
                }
                else
                {
                    std::cout << "cannot open file to write cloud." << std::endl;
                }
            }
        }
        usleep(500);
    }
    sock.uninit();
    return true;
}

// 初始化velodyne设备并获取点云
#include "../velodyne_lidar/velodyne_driver/velodyne_lidar_device.h"
void device_test()
{
    velodyne::velodyneLidarParams params;
    params.set_ip("");
    params.set_port(2368);
    params.set_model("VLP16");
    params.set_calibrationfile("/home/youchen/extra_space/chutian/measure/puai_wj_2021/setting/VLP16db.yaml");
    params.mutable_calib()->set_y(1.57);

    Velodyne_lidar_device t_device;
    t_device.init(params);

    usleep(1000 * 1000);

    std::mutex p_mutex;
    pcl::PointCloud<pcl::PointXYZ>::Ptr p_cloud(new pcl::PointCloud<pcl::PointXYZ>);
    Error_manager ec = t_device.get_new_cloud_and_wait(&p_mutex, p_cloud, std::chrono::steady_clock::now(), 6000);

    if (ec == SUCCESS)
    {
        std::ofstream out("/home/youchen/extra_space/chutian/measure/puai_wj_2021/build/device_point.txt");
        if (out.is_open())
        {
            for (size_t i = 0; i < p_cloud->size(); i++)
            {
                char buf[255] = {0};
                sprintf(buf, "%.3f %.3f %.3f\n", p_cloud->points[i].x, p_cloud->points[i].y, p_cloud->points[i].z);
                out << buf;
            }
            std::cout << "cloud written" << std::endl;
            out.close();
        }
        else
        {
            std::cout << "cannot open file to write cloud." << std::endl;
        }
    }else{
        std::cout << "get cloud failed" << std::endl;
    }
    t_device.uninit();
}

#include "../velodyne_lidar/ground_region.h"
#include "../tool/measure_filter.h"
#include "../tool/point_tool.h"
void region_detect()
{
    std::ifstream in("/home/youchen/extra_space/chutian/measure/puai_wj_2021/build/comb_calib.txt");
    if(!in.is_open())
    {
        std::cout << "cannot open file to read cloud." << std::endl;
        return;
    }
    else
    {
        // 准备点云
        pcl::PointCloud<pcl::PointXYZ>::Ptr t_cloud(new pcl::PointCloud<pcl::PointXYZ>);
        while(!in.eof())
        {
            std::string t_linestr;
            if (getline(in, t_linestr))
            {
                std::vector<std::string> str_vector;
                std::stringstream ss(t_linestr);
                std::string num_str;
                while (getline(ss, num_str, ' '))
                {
                    str_vector.push_back(num_str);
                }
                if (str_vector.size() != 3)
                {
                    std::cout << "unsupported point cloud / cannot find point x y z value " << std::endl;
                    return;
                }
                pcl::PointXYZ t_point;
                t_point.x = stod(str_vector[0]);
                t_point.y = stod(str_vector[1]);
                t_point.z = stod(str_vector[2]);
                t_cloud->push_back(t_point);
            }
        }
        // 初始化region
        velodyne::Region param;
        param.set_minx(-3.10);
        param.set_maxx(0);
        param.set_miny(-2.75);
        param.set_maxy(3.08);
        param.set_minz(0.01);
        param.set_maxz(0.2);
        param.set_region_id(0);
        pcl::PointCloud<pcl::PointXYZ>::Ptr left = pcl::PointCloud<pcl::PointXYZ>::Ptr(new pcl::PointCloud<pcl::PointXYZ>);
        pcl::PointCloud<pcl::PointXYZ>::Ptr right = pcl::PointCloud<pcl::PointXYZ>::Ptr(new pcl::PointCloud<pcl::PointXYZ>);
        if (!read_pointcloud("/home/youchen/extra_space/chutian/measure/puai_wj_2021/setting/left_model.txt", left) || !read_pointcloud("/home/youchen/extra_space/chutian/measure/puai_wj_2021/setting/right_model.txt", right))
        {
            std::cout << "cannot read left/right model " << std::endl;
            return;
        }
        Ground_region t_region;
        std::cout << "before init" << std::endl;
        t_region.init(param, left, right);
        std::cout << "after init" << std::endl;
        // 测试region功能
        detect_wheel_ceres3d::Detect_result t_result;
        Error_manager ec = t_region.detect(t_cloud, t_result);
        std::cout << ec.to_string() << std::endl;
        std::cout << "---------------" << std::endl;
        for (size_t i = 0; i < 12; i++)
        {
            t_region.update_cloud(t_cloud);
            usleep(1000*100);
        }

        Common_data::Car_wheel_information t_wheel_info;

        ec = Measure_filter::get_instance_references().get_filtered_wheel_information(0, t_wheel_info);
        if (ec == SUCCESS)
        {
            std::cout << t_wheel_info.to_string() << std::endl;
        }
        else { std::cout << ec.to_string() << std::endl; }
    }
}

int main()
{
    // velo_driver_test();
    // device_test();
    region_detect();

    return 0;
}
